import { useState, useEffect, useCallback, useRef, useMemo, } from "react";
const usePlacement = ({ isOpen, defaultPlacement = "bottom", defaultAlignment = "start", filter = "auto", strategy = "absolute", allowedPlacements = [], allowedAlignments = [], }) => {
    const [placement, setPlacement] = useState(defaultPlacement);
    const [alignment, setAlignment] = useState(defaultAlignment);
    const referenceRef = useRef(null);
    const contentRef = useRef(null);
    const gapRatio = useRef(0.1);
    const isMount = useRef(false);
    useEffect(() => {
        isMount.current = true;
        return () => {
            isMount.current = false;
        };
    }, []);
    const calculatePlacement = useCallback(() => {
        const reference = referenceRef.current;
        const content = contentRef.current;
        if (!reference || !content)
            return;
        const referenceRect = reference.getBoundingClientRect();
        const contentRect = content.getBoundingClientRect();
        const viewportWidth = window.innerWidth;
        const viewportHeight = window.innerHeight;
        // const {height, width} = referenceRect;
        const verticalGap = referenceRect.height * gapRatio.current;
        const horizontalGap = referenceRect.width * gapRatio.current;
        const checkPositions = {
            top: {
                space: referenceRect.top - contentRect.height - verticalGap,
                align: {
                    center: referenceRect.left + referenceRect.width / 2 - contentRect.width / 2 >= 0 &&
                        referenceRect.right + contentRect.width / 2 <= viewportWidth,
                    start: referenceRect.left + contentRect.width <= viewportWidth,
                    end: referenceRect.right - contentRect.width >= 0,
                },
            },
            right: {
                space: viewportWidth - referenceRect.right - contentRect.width - horizontalGap,
                align: {
                    center: referenceRect.top + referenceRect.height / 2 - contentRect.height / 2 >= 0 &&
                        referenceRect.bottom + contentRect.height / 2 <= viewportHeight,
                    start: referenceRect.top + contentRect.height <= viewportHeight,
                    end: referenceRect.bottom - contentRect.height >= 0,
                },
            },
            bottom: {
                space: viewportHeight - referenceRect.bottom - contentRect.height - verticalGap,
                align: {
                    center: referenceRect.left + referenceRect.width / 2 - contentRect.width / 2 >= 0 &&
                        referenceRect.right + contentRect.width / 2 <= viewportWidth,
                    start: referenceRect.left + contentRect.width <= viewportWidth,
                    end: referenceRect.right - contentRect.width >= 0,
                },
            },
            left: {
                space: referenceRect.left - contentRect.width - horizontalGap,
                align: {
                    center: referenceRect.top + referenceRect.height / 2 - contentRect.height / 2 >= 0 &&
                        referenceRect.bottom + contentRect.height / 2 <= viewportHeight,
                    start: referenceRect.top + contentRect.height <= viewportHeight,
                    end: referenceRect.bottom - contentRect.height >= 0,
                },
            },
        };
        const placementOrder = getPlacementOrder(defaultPlacement, allowedPlacements);
        const alignmentOrder = getAlignmentOrder(defaultAlignment, allowedAlignments);
        for (const pos of placementOrder) {
            const position = checkPositions[pos];
            if (position.space > 0) {
                const validAlignment = alignmentOrder.find((align) => position.align[align]);
                if (validAlignment) {
                    setPlacement(pos);
                    setAlignment(validAlignment);
                    return;
                }
            }
        }
        setPlacement(defaultPlacement);
        setAlignment(defaultAlignment);
    }, [defaultPlacement, defaultAlignment]);
    useEffect(() => {
        if (!isMount.current || !isOpen)
            return;
        calculatePlacement();
        if (filter === "static")
            return;
        const handleResize = () => calculatePlacement();
        const resizeObserver = new ResizeObserver(calculatePlacement);
        window.addEventListener("resize", handleResize);
        window.addEventListener("scroll", handleResize, true);
        if (referenceRef.current)
            resizeObserver.observe(referenceRef.current);
        if (contentRef.current)
            resizeObserver.observe(contentRef.current);
        return () => {
            window.removeEventListener("resize", handleResize);
            window.removeEventListener("scroll", handleResize, true);
            resizeObserver.disconnect();
        };
    }, [isOpen, filter, calculatePlacement]);
    const getPositionStyles = useCallback(() => {
        if (filter === "static") {
            setPlacement(defaultPlacement);
            setAlignment(defaultAlignment);
        }
        const reference = referenceRef.current;
        if (!isMount.current || !reference) {
            return {
                position: strategy,
                display: "none",
            };
        }
        const referenceRect = reference.getBoundingClientRect();
        const verticalGap = referenceRect.height * gapRatio.current;
        const horizontalGap = referenceRect.width * gapRatio.current;
        const baseStyles = {
            position: strategy,
            display: isOpen ? "block" : "none",
            transition: "opacity 0.2s ease",
        };
        switch (placement) {
            case "top":
                return alignment === "center"
                    ? Object.assign(Object.assign({}, baseStyles), { bottom: `calc(100% + ${verticalGap}px)`, left: "50%", transform: "translateX(-50%)" }) : Object.assign(Object.assign({}, baseStyles), { bottom: `calc(100% + ${verticalGap}px)`, [alignment === "start" ? "left" : "right"]: 0 });
            case "right":
                if (alignment === 'start') {
                    return Object.assign(Object.assign({}, baseStyles), { left: `calc(100% + ${horizontalGap}px)`, bottom: 0 });
                }
                else if (alignment === "center") {
                    return Object.assign(Object.assign({}, baseStyles), { left: `calc(100% + ${horizontalGap}px)`, top: "50%", transform: "translateY(-50%)" });
                }
                else if (alignment === 'end') {
                    return Object.assign(Object.assign({}, baseStyles), { left: `calc(100% + ${horizontalGap}px)`, top: 0 });
                }
                return alignment === "center"
                    ? Object.assign(Object.assign({}, baseStyles), { left: `calc(100% + ${horizontalGap}px)`, top: "50%", transform: "translateY(-50%)" }) : Object.assign(Object.assign({}, baseStyles), { left: `calc(100% + ${horizontalGap}px)`, [alignment === "start" ? "top" : "bottom"]: 0 });
            case "bottom":
                return alignment === "center"
                    ? Object.assign(Object.assign({}, baseStyles), { top: `calc(100% + ${verticalGap}px)`, left: "50%", transform: "translateX(-50%)" }) : Object.assign(Object.assign({}, baseStyles), { top: `calc(100% + ${verticalGap}px)`, [alignment === "start" ? "left" : "right"]: 0 });
            case "left":
                if (alignment === 'start') {
                    return Object.assign(Object.assign({}, baseStyles), { right: `calc(100% + ${horizontalGap}px)`, bottom: 0 });
                }
                else if (alignment === "center") {
                    return Object.assign(Object.assign({}, baseStyles), { right: `calc(100% + ${horizontalGap}px)`, top: "50%", transform: "translateY(-50%)" });
                }
                else if (alignment === 'end') {
                    return Object.assign(Object.assign({}, baseStyles), { right: `calc(100% + ${horizontalGap}px)`, top: 0 });
                }
                return alignment === "center"
                    ? Object.assign(Object.assign({}, baseStyles), { right: `calc(100% + ${horizontalGap}px)`, top: "50%", transform: "translateY(-50%)" }) : Object.assign(Object.assign({}, baseStyles), { right: `calc(100% + ${horizontalGap}px)`, [alignment === "start" ? "top" : "bottom"]: 0 });
        }
    }, [placement, alignment, strategy, isOpen, filter, defaultPlacement, defaultAlignment]);
    const styles = useMemo(() => getPositionStyles(), [getPositionStyles]);
    return {
        placement,
        alignment,
        strategy,
        styles,
        refs: {
            reference: referenceRef,
            content: contentRef,
        },
    };
};
const getPlacementOrder = (defaultPlacement, allowedPlacements) => {
    const allPlacements = ["top", "right", "bottom", "left"];
    if (!(allowedPlacements === null || allowedPlacements === void 0 ? void 0 : allowedPlacements.length))
        return allPlacements;
    const filteredPlacements = allPlacements.filter((p) => allowedPlacements.includes(p));
    if (!filteredPlacements.length)
        return [defaultPlacement];
    const defaultIndex = filteredPlacements.indexOf(defaultPlacement);
    if (defaultIndex === -1)
        return filteredPlacements;
    return [
        ...filteredPlacements.slice(defaultIndex),
        ...filteredPlacements.slice(0, defaultIndex),
    ];
};
const getAlignmentOrder = (defaultAlignment, allowedAlignments) => {
    const allAlignments = ["start", "center", "end"];
    if (!(allowedAlignments === null || allowedAlignments === void 0 ? void 0 : allowedAlignments.length))
        return allAlignments;
    const filteredAlignments = allAlignments.filter((a) => allowedAlignments.includes(a));
    if (!filteredAlignments.length)
        return [defaultAlignment];
    const defaultIndex = filteredAlignments.indexOf(defaultAlignment);
    if (defaultIndex === -1)
        return filteredAlignments;
    return [
        ...filteredAlignments.slice(defaultIndex),
        ...filteredAlignments.slice(0, defaultIndex),
    ];
};
export default usePlacement;
